const { pool } = require("../config/database/database.config");
const { getStateId } = require("./stateRepository");
/*
Add address info into address table and peopleXaddress table
*/
async function addAddress(address, peopleId) {
  // get all the parts of the address
  let { address1, address2, city, state, country, zipCode, preferredAddress } =
    address;

  console.log("adding address...");

  try {
    // Use helper functions to look up stateId and countryId
    const stateId = await getStateId(state);
    const countryId = await _lookupCountryId(country);

    // query for adding the address
    const query = `
      INSERT INTO address (address1, address2, city, stateId, countryId, zipCode)
      VALUES (?,?,?,?,?,?)
    `;

    // insert data into the address table
    const [addressResult] = await pool.query(query, [
      address1,
      address2,
      city,
      stateId,
      countryId,
      zipCode,
    ]);

    // get the id of the newly created address in the DB
    const addressId = addressResult.insertId;

    console.log(
      "successfully inserted addressID " +
        addressId +
        " for person: " +
        peopleId
    );

    // Now insert into the peopleXAddress table
    const peopleAddressQuery = `
        INSERT INTO peopleXAddress (peopleId, addressId, preferredAddress)
        VALUES (?, ?, ?)
    `;
    const [peopleAddressResult] = await pool.query(peopleAddressQuery, [
      peopleId,
      addressId,
      preferredAddress,
    ]);

    console.log("Added into peopleXaddress");

    return peopleAddressResult;
  } catch (error) {
    // return null if error
    console.error(error);
    return null;
  }
}

/*
Function to update an address. If the address is successfully updated,
the function returns the updated address's ID. If the address is not successfully
updated, the function returns null
*/
async function updateAddress(addressId, peopleId, updatedAddress) {
  // get the parts of the address
  let { address1, address2, city, state, country, zipCode, preferredAddress } =
    updatedAddress;

  console.log("Updating address...");

  try {
    // look up the state and country
    const stateId = await getStateId(state);
    const countryId = await _lookupCountryId(country);

    // Update the main address table
    const updateAddressQuery = `
      UPDATE address
      SET address1 = ?, address2 = ?, city = ?, stateId = ?, countryId = ?, zipCode = ?
      WHERE addressId = ?
    `;
    const [addressResult] = await pool.query(updateAddressQuery, [
      address1,
      address2,
      city,
      stateId,
      countryId,
      zipCode,
      addressId,
    ]);

    // Check if any rows were updated in the address table
    if (addressResult.affectedRows === 0) {
      console.error("Address not found. No rows updated in the address table.");
      return null;
    }
    console.log("Address table updated successfully.");

    // Update the preferredAddress in peopleXAddress if provided
    if (preferredAddress !== undefined) {
      const updatePeopleXAddressQuery = `
        UPDATE peopleXAddress
        SET preferredAddress = ?
        WHERE peopleId = ? AND addressId = ?
      `;

      const [peopleXAddressResult] = await pool.query(
        updatePeopleXAddressQuery,
        [preferredAddress, peopleId, addressId]
      );

      // Check if any rows were updated in the peopleXAddress table
      if (peopleXAddressResult.affectedRows === 0) {
        console.error(
          "Associated record in peopleXAddress not found. No rows updated in the peopleXAddress table."
        );
        return null;
      }

      console.log("peopleXAddress table updated successfully.");
    }

    return addressId;
  } catch (error) {
    console.log("Error updating address: ", error);
    return null;
  }
}

/*
Function to delete an address by its ID. 
Deletes from both peopleXAddress and address tables.
Returns true if the deletion was successful and false if no rows were affected.
*/
async function deleteAddressById(addressId) {
  console.log("Deleting address with ID: ", addressId);

  try {
    // delete from the peopleXAddress table to satisfy any foreign key constraints
    const deletePeopleXAddressQuery = `
      DELETE FROM peopleXAddress
      WHERE addressId = ?
    `;
    const [peopleXAddressResult] = await pool.query(deletePeopleXAddressQuery, [
      addressId,
    ]);

    // Check if any rows were affected in peopleXAddress
    if (peopleXAddressResult.affectedRows === 0) {
      console.error(
        "No associated record found in peopleXAddress. No rows deleted."
      );
      return false;
    }

    console.log("Deleted from peopleXAddress table");

    // delete the address from the address table
    const deleteAddressQuery = `
      DELETE FROM address
      WHERE addressId = ?
    `;
    const [addressResult] = await pool.query(deleteAddressQuery, [addressId]);

    // Check if any rows were affected in the address table
    if (addressResult.affectedRows === 0) {
      console.error("Address not found in address table. No rows deleted.");
      return false;
    }

    console.log("Address deleted successfully from address table.");

    return true;
  } catch (error) {
    console.error("Error deleting address: ", error);
    return false;
  }
}

/*
Function to find all address information associated with a person.
Returns an array of address objects or an empty array if no addresses exist.
*/
async function getAddressesForPerson(peopleId) {
  console.log("Finding addresses for person with ID: ", peopleId);

  try {
    const query = `
      SELECT 
        a.addressId, 
        a.address1, 
        a.address2, 
        a.city, 
        sl.name AS state, 
        cl.name AS country, 
        a.zipCode, 
        pa.preferredAddress
      FROM 
        peopleXAddress pa
      LEFT JOIN 
        address a ON pa.addressId = a.addressId
      LEFT JOIN 
        stateLookup sl ON a.stateId = sl.stateId
      LEFT JOIN 
        countryLookup cl ON a.countryId = cl.countryId
      WHERE 
        pa.peopleId = ?;
    `;

    // Execute the query
    const [results] = await pool.query(query, [peopleId]);

    if (results.length === 0) {
      console.log("No addresses found for person with ID:", peopleId);
      return [];
    }

    // Map the results to structured address objects
    const addresses = results.map((row) => ({
      addressId: row.addressId,
      address1: row.address1,
      address2: row.address2,
      city: row.city,
      state: row.state,
      country: row.country,
      zipCode: row.zipCode,
      preferredAddress: row.preferredAddress,
    }));

    console.log("Found addresses: ", addresses);

    return addresses;
  } catch (error) {
    console.error("Error fetching addresses for person: ", error);
    return [];
  }
}

// Private helper function to look up the countryId
async function _lookupCountryId(country) {
  // for now return 1 while tables are not populated
  return 1;
}

module.exports = {
  addAddress,
  getAddressesForPerson,
  deleteAddressById,
  updateAddress,
};
